@@ -1,7 +1,6 @@
 #!/bin/sh
 # Run this to set up the build system: configure, makefiles, etc.
 
-# $Id: autogen.sh,v 1.3.4.1 2002/01/24 01:30:56 giles Exp $
 
 package="ghostscript"
 
@@ -21,17 +20,12 @@ cd "$srcdir"
 echo "Generating configuration files for $package, please wait...."
 
 echo "  creating links to configuration sources"
-ln -sf src/configure.ac .
-ln -sf src/Makefile.in .
+rm -f configure.ac Makefile.in
+ln -s base/configure.ac .
+ln -s base/Makefile.in .
 
-#echo "  aclocal $ACLOCAL_FLAGS"
-#aclocal $ACLOCAL_FLAGS
-#echo "  autoheader"
-#autoheader
-#echo "  automake --add-missing $AUTOMAKE_FLAGS"
-#automake --add-missing $AUTOMAKE_FLAGS 
 echo "  running autoconf"
-autoconf
+autoconf || exit 1
 
 if test -z "$*"; then
         echo "I am going to run ./configure with no arguments - if you wish "
