@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
-  
+
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gstrans.h 11947 2010-12-10 19:50:53Z mvrhel $ */
+
 /* Transparency definitions and interface */
 
 #ifndef gstrans_INCLUDED
@@ -89,10 +91,10 @@ struct gs_pdf14trans_params_s {
     int Background_components;
     bool function_is_identity;
     float Background[GS_CLIENT_COLOR_MAX_COMPONENTS];
-    float GrayBackground;  /* This is used to determine if the 
+    float GrayBackground;  /* This is used to determine if the
                               softmask's bbox needs to be adjusted
                               to the parent groups bbox.  Since
-                              the soft mask can affect areas 
+                              the soft mask can affect areas
                               outside its own groups bounding
                               box in such a case */
     gs_function_t *transfer_function;
@@ -105,8 +107,8 @@ struct gs_pdf14trans_params_s {
     bool mask_is_image;
     gs_matrix ctm;
     bool replacing;
-	bool overprint;
-	bool overprint_mode;
+    bool overprint;
+    bool overprint_mode;
     bool idle; /* For clist reader.*/
     uint mask_id; /* For clist reader.*/
     int group_color_numcomps;
@@ -115,7 +117,7 @@ struct gs_pdf14trans_params_s {
     cmm_profile_t *iccprofile;               /* The profile  */
     bool crop_blend_params;  /* This is used when the blend params are updated
                                 during a transparency group push */
-
+    bool is_pattern;      /* Needed to detect device push and pop for clist pattern */
 };
 
 #ifndef gs_pdf14trans_params_DEFINED
@@ -133,7 +135,6 @@ typedef struct gs_pdf14trans_s {
     gs_pdf14trans_params_t  params;
 } gs_pdf14trans_t;
 
-
 /* Access transparency-related graphics state elements. */
 int gs_setblendmode(gs_state *, gs_blend_mode_t);
 gs_blend_mode_t gs_currentblendmode(const gs_state *);
@@ -156,27 +157,29 @@ gs_transparency_state_type_t
  * We have to abbreviate the procedure name because procedure names are
  * only unique to 23 characters on VMS.
  */
-int gs_push_pdf14trans_device(gs_state * pgs);
+int gs_push_pdf14trans_device(gs_state * pgs, bool is_pattern);
 
-int gs_pop_pdf14trans_device(gs_state * pgs);
+int gs_pop_pdf14trans_device(gs_state * pgs, bool is_pattern);
 
 void gs_trans_group_params_init(gs_transparency_group_params_t *ptgp);
 
+int gs_update_trans_marking_params(gs_state * pgs);
+
 int gs_begin_transparency_group(gs_state * pgs,
-				const gs_transparency_group_params_t *ptgp,
-				const gs_rect *pbbox);
+                                const gs_transparency_group_params_t *ptgp,
+                                const gs_rect *pbbox);
 
 int gs_end_transparency_group(gs_state *pgs);
 
 void gs_trans_mask_params_init(gs_transparency_mask_params_t *ptmp,
-			       gs_transparency_mask_subtype_t subtype);
+                               gs_transparency_mask_subtype_t subtype);
 
 int gs_begin_transparency_mask(gs_state *pgs,
-			       const gs_transparency_mask_params_t *ptmp,
-			       const gs_rect *pbbox, bool mask_is_image);
+                               const gs_transparency_mask_params_t *ptmp,
+                               const gs_rect *pbbox, bool mask_is_image);
 
 int gs_end_transparency_mask(gs_state *pgs,
-			     gs_transparency_channel_selector_t csel);
+                             gs_transparency_channel_selector_t csel);
 
 int gs_discard_transparency_layer(gs_state *pgs);
 
@@ -184,19 +187,19 @@ int gs_discard_transparency_layer(gs_state *pgs);
  * Imager level routines for the PDF 1.4 transparency operations.
  */
 int gx_begin_transparency_group(gs_imager_state * pis, gx_device * pdev,
-				const gs_pdf14trans_params_t * pparams);
+                                const gs_pdf14trans_params_t * pparams);
 
 int gx_end_transparency_group(gs_imager_state * pis, gx_device * pdev);
 
 int gx_begin_transparency_mask(gs_imager_state * pis, gx_device * pdev,
-				const gs_pdf14trans_params_t * pparams);
+                                const gs_pdf14trans_params_t * pparams);
 
 int gx_end_transparency_mask(gs_imager_state * pis, gx_device * pdev,
-				const gs_pdf14trans_params_t * pparams);
+                                const gs_pdf14trans_params_t * pparams);
 
 int gx_discard_transparency_layer(gs_imager_state *pis);
 
-/* These are used for watching for q Smask Q events.  We need to 
+/* These are used for watching for q Smask Q events.  We need to
    send special compositor commands to keep the bands in sync
    with the current softmask during clist rendering.  Like the
    other transparency operations the gs functions occur on the
@@ -205,13 +208,12 @@ int gx_discard_transparency_layer(gs_imager_state *pis);
 
 int gs_push_transparency_state(gs_state *pgs);
 
-int gs_pop_transparency_state(gs_state *pgs);
+int gs_pop_transparency_state(gs_state *pgs, bool force);
 
 int gx_push_transparency_state(gs_imager_state * pis, gx_device * pdev);
 
 int gx_pop_transparency_state(gs_imager_state * pis, gx_device * pdev);
 
-
 /*
  * Verify that a compositor data structure is for the PDF 1.4 compositor.
  */
@@ -230,15 +232,15 @@ int gs_is_pdf14trans_compositor(const gs_composite_t * pct);
  * since it is a hack, we may exceed our desired buffer space while
  * processing the file.
  */
-#define NUM_PDF14_BUFFERS 3
-#define NUM_ALPHA_CHANNELS 1
-#define NUM_COLOR_CHANNELS 4
-#define BITS_PER_CHANNEL 8
+#define NUM_PDF14_BUFFERS 3     /* totally a random guess */
+#define NUM_ALPHA_CHANNELS 1    /* common, but doesn't include possible tag, shape or group alpha */
+#define NUM_COLOR_CHANNELS 4    /* CMYK is most common 'worst case' */
+#define BITS_PER_CHANNEL 8      /* currently pdf14 device is always 8-bit */
 /* The estimated size of an individual PDF 1.4 buffer row (in bits) */
-#define ESTIMATED_PDF14_ROW_SIZE(width) ((width) * BITS_PER_CHANNEL\
-	* (NUM_ALPHA_CHANNELS + NUM_COLOR_CHANNELS))
+#define ESTIMATED_PDF14_ROW_SIZE(width, target_num_components) ((width) * BITS_PER_CHANNEL\
+        * (NUM_ALPHA_CHANNELS + NUM_COLOR_CHANNELS))
 /* The estimated size of one row in all PDF 1.4 buffers (in bits) */
-#define ESTIMATED_PDF14_ROW_SPACE(width) \
-	(NUM_PDF14_BUFFERS * ESTIMATED_PDF14_ROW_SIZE(width))
+#define ESTIMATED_PDF14_ROW_SPACE(width, target_num_components) \
+        (NUM_PDF14_BUFFERS * ESTIMATED_PDF14_ROW_SIZE(width, target_num_components))
 
 #endif /* gstrans_INCLUDED */
