@@ -1,21 +1,20 @@
 #!/usr/bin/tclsh
 
-#    Copyright (C) 2000-2002 Artifex Software, Inc. All rights reserved.
-# 
+# Copyright (C) 2001-2012 Artifex Software, Inc.
+# All Rights Reserved.
+#
 # This software is provided AS-IS with no warranty, either express or
 # implied.
-# 
+#
 # This software is distributed under license and may not be copied,
 # modified or distributed except as expressly authorized under the terms
 # of the license contained in the file LICENSE in this distribution.
-# 
-# For more information about licensing, please refer to
-# http://www.ghostscript.com/licensing/. For information on
-# commercial licensing, go to http://www.artifex.com/licensing/ or
-# contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-# San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+#
+# Refer to licensing information at http://www.artifex.com or contact
+# Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+# CA  94903, U.S.A., +1(415)492-9861, for further information.
+#
 
-# $Id: pre.tcl 8383 2007-11-19 20:23:39Z giles $
 
 # Check various aspects of an about-to-be-released Ghostscript fileset.
 # Only applicable to filesets 6.0 and later (assumes use of SVN).
@@ -24,7 +23,7 @@
 set DIFFFILE t
 set GREPFILE t1
 set CHECKFILE /gs/toolbin/pre.chk
-set GSINIT lib/gs_init.ps
+set GSINIT Resource/Init/gs_init.ps
 set NEWS doc/News.htm
 
 # ---------------- Utilities ---------------- #
@@ -154,11 +153,33 @@ foreach d "[glob doc/*.htm] doc/gs-vms.hlp" {
     }
 }
 
+switch $tcl_platform(platform) {
+     unix {
+         # or even $::env(TMPDIR), at times.
+         set tmpdir /tmp
+     } macintosh {
+         set tmpdir $::env(TRASH_FOLDER)  ;# a better place?
+     } default {
+         set tmpdir [pwd]
+         catch {set tmpdir $::env(TMP)}
+         catch {set tmpdir $::env(TEMP)}
+     }
+}
+
 if {$argv == {update}} {
-	# Update dates in .htm and .1 files.
+    # Update dates in .htm and .1 files.
     proc updoc {doc before after} {
-	set tmpfile /tmp/[pid]
-	catch {file delete $tmpfile}
+	set tmpfile [file join /tmp [pid]]
+	set access [list RDWR CREAT EXCL TRUNC]
+	set perm 0600
+	if {[catch {open $tmpfile $access $perm} fid ]} {
+	    # something went wrong
+	     error "Could not open tempfile."
+	}
+	if {[catch {close $fid} err]} {
+	    error "Failed closing temporary file: $err"
+	}
+
 	exec perl -pwe "s{$before}{$after}" < $doc > $tmpfile
 	file rename -force $tmpfile $doc
     }
@@ -185,7 +206,7 @@ exec touch $DIFFFILE
 
 ################ Check dates and version #s in makefiles
 
-check_version src/version.mak
+check_version base/version.mak
 
 ################ Check dates and version #s in documentation files
 
@@ -223,7 +244,7 @@ foreach doc $manlist {
     # We must be careful not to include the string $,I,d,: in any pattern,
     # since CVS/RCS will substitute for it!
     if {![regexp {^\.\\" [$]Id: ([^ ]+) ([0-9.]+) ([0-9][0-9][0-9][0-9])/([0-9][0-9])/([0-9][0-9])} $idline skip file idrevision idyear idmonth idday]} {
-	message "In $doc, can't parse $Id line: $idline"
+	message "In $doc, can't parse \$Id line: $idline"
 	continue
     }
     if {$file != [file tail $doc]} {
@@ -280,9 +301,9 @@ proc grept1 {pattern files {force_filename 1}} {
     }
     catch {eval exec grep $pattern $files $t >> $GREPFILE}
 }
-set unix_maks {src/unixansi.mak src/unixtrad.mak src/unix-gcc.mak src/dvx-gcc.mak}
-set pc_maks {src/bcwin32.mak src/msvc32.mak src/watc.mak src/watcwin.mak}
-set lib_maks {src/msvclib.mak src/watclib.mak src/ugcclib.mak}
+set unix_maks {base/unixansi.mak base/unixtrad.mak base/unix-gcc.mak base/dvx-gcc.mak}
+set pc_maks {base/bcwin32.mak base/msvc32.mak base/watc.mak base/watcwin.mak}
+set lib_maks {base/msvclib.mak base/watclib.mak base/ugcclib.mak}
 set all_maks [concat $unix_maks $pc_maks $lib_maks]
 grept1 ^GENOPT= $unix_maks
 grept1 ^CFLAGS= $unix_maks
@@ -293,23 +314,23 @@ grept1 ^ZLIB_NAME= $unix_maks
 grept1 ^DEBUG= $pc_maks
 grept1 ^TDEBUG= $pc_maks
 grept1 ^NOPRIVATE= $pc_maks
-grept1 bgi_= src/devs.mak
-grept1 mswin_= src/devs.mak
+grept1 bgi_= base/devs.mak
+grept1 mswin_= base/devs.mak
 grept1 ^BAND_LIST_ $all_maks
 grept1 ^COMPILE_INITS= $unix_maks
 grept1 DEVICE_DEVS $all_maks
 grept1 FEATURE_DEVS $all_maks
 grept1 ^SHARE_ $unix_maks
 grept1 ^..._TYPE= $pc_maks
-grept1 define._fixed_shift src/gxfixed.h
-grept1 define.TRIM src/gdevbit.c
-grept1 define.EXPAND src/gdevbit.c
-grept1 if...1...return.0 src/gdevxalt.c
-grept1 define.TEST [glob src/gdev*.c]
-grept1 define.PDFX src/gdevpdfx.h
-grept1 define.FORCE src/gshtscr.c
-grept1 define.CAPTURE src/gslib.c
-grept1 interval.= src/zcontext.c
+grept1 define._fixed_shift base/gxfixed.h
+grept1 define.TRIM base/gdevbit.c
+grept1 define.EXPAND base/gdevbit.c
+grept1 if...1...return.0 base/gdevxalt.c
+grept1 define.TEST [glob base/gdev*.c]
+grept1 define.PDFX base/gdevpdfx.h
+grept1 define.FORCE base/gshtscr.c
+grept1 define.CAPTURE base/gslib.c
+grept1 interval.= base/zcontext.c
 # Apparently diff exits with a non-zero status if there are any differences!
 catch {exec diff $GREPFILE $CHECKFILE >> $DIFFFILE}
 exec wc $DIFFFILE >@ stdout
